# API Docs - v4.0.1

## Store

### rdbms *<a target="_blank" href="https://wso2.github.io/siddhi/documentation/siddhi-4.0/#store">(Store)</a>*

<p style="word-wrap: break-word">This extension assigns data sources and connection instructions to event tables. It also implements read write operations on connected datasources</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
@Store(type="rdbms", jdbc.url="<STRING>", username="<STRING>", password="<STRING>", jdbc.driver.name="<STRING>", pool.properties="<STRING>", jndi.resource="<STRING>", table.name="<STRING>", field.length="<STRING>")
@PrimaryKey("PRIMARY_KEY")
@Index("INDEX")
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">jdbc.url</td>
        <td style="vertical-align: top; word-wrap: break-word">The JDBC URL via which the RDBMS data store is accessed.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">username</td>
        <td style="vertical-align: top; word-wrap: break-word">The username to be used to access the RDBMS data store.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">password</td>
        <td style="vertical-align: top; word-wrap: break-word">The password to be used to access the RDBMS data store.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">jdbc.driver.name</td>
        <td style="vertical-align: top; word-wrap: break-word">The driver class name for connecting the RDBMS data store.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">pool.properties</td>
        <td style="vertical-align: top; word-wrap: break-word">Any pool parameters for the database connection must be specified as key value pairs.</td>
        <td style="vertical-align: top">null</td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">jndi.resource</td>
        <td style="vertical-align: top; word-wrap: break-word">The name of the JNDI resource through which the connection is attempted. If this is found, the pool properties described above are not taken into account, and the connection is attempted via JNDI lookup instead.</td>
        <td style="vertical-align: top">null</td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">table.name</td>
        <td style="vertical-align: top; word-wrap: break-word">The name with which the event table should be persisted in the store. If no name is specified via this parameter, the event table is persisted with the same name as the Siddhi table.</td>
        <td style="vertical-align: top">The table name defined in the Siddhi App query.</td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">field.length</td>
        <td style="vertical-align: top; word-wrap: break-word">The number of characters that the values for fields of the <code>STRING</code> type in the table definition must contain. If this is not specified, the default number of characters specific to the database type is considered.</td>
        <td style="vertical-align: top">null</td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="system-parameters" class="md-typeset" style="display: block; font-weight: bold;">System Parameters</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Parameters</th>
    </tr>
    <tr>
        <td style="vertical-align: top">{{RDBMS-Name}}.maxVersion</td>
        <td style="vertical-align: top; word-wrap: break-word">The latest version supported for {{RDBMS-Name}}.</td>
        <td style="vertical-align: top">0</td>
        <td style="vertical-align: top">N/A</td>
    </tr>
    <tr>
        <td style="vertical-align: top">{{RDBMS-Name}}.minVersion</td>
        <td style="vertical-align: top; word-wrap: break-word">The earliest version supported for {{RDBMS-Name}}.</td>
        <td style="vertical-align: top">0</td>
        <td style="vertical-align: top">N/A</td>
    </tr>
    <tr>
        <td style="vertical-align: top">{{RDBMS-Name}}.tableCheckQuery</td>
        <td style="vertical-align: top; word-wrap: break-word">The template query for the <code>check table</code> operation in {{RDBMS-Name}}.</td>
        <td style="vertical-align: top"><b>H2</b>: CREATE TABLE {{TABLE_NAME}} ({{COLUMNS, PRIMARY_KEYS}})<br><b>MySQL</b>: CREATE TABLE {{TABLE_NAME}} ({{COLUMNS, PRIMARY_KEYS}})<br><b>Oracle</b>: CREATE TABLE {{TABLE_NAME}} ({{COLUMNS, PRIMARY_KEYS}})<br><b>Microsoft SQL Server</b>: CREATE TABLE {{TABLE_NAME}} ({{COLUMNS, PRIMARY_KEYS}})<br><b>PostgreSQL</b>: CREATE TABLE {{TABLE_NAME}} ({{COLUMNS, PRIMARY_KEYS}})<br><b>DB2.*</b>: CREATE TABLE {{TABLE_NAME}} ({{COLUMNS, PRIMARY_KEYS}})</td>
        <td style="vertical-align: top">N/A</td>
    </tr>
    <tr>
        <td style="vertical-align: top">{{RDBMS-Name}}.tableCreateQuery</td>
        <td style="vertical-align: top; word-wrap: break-word">The template query for the <code>create table</code> operation in {{RDBMS-Name}}.</td>
        <td style="vertical-align: top"><b>H2</b>: SELECT 1 FROM {{TABLE_NAME}} LIMIT 1<br><b>MySQL</b>: SELECT 1 FROM {{TABLE_NAME}} LIMIT 1<br><b>Oracle</b>: SELECT 1 FROM {{TABLE_NAME}} WHERE rownum=1<br><b>Microsoft SQL Server</b>: SELECT TOP 1 1 from {{TABLE_NAME}}<br><b>PostgreSQL</b>: SELECT 1 FROM {{TABLE_NAME}} LIMIT 1<br><b>DB2.*</b>: SELECT 1 FROM {{TABLE_NAME}} FETCH FIRST 1 ROWS ONLY</td>
        <td style="vertical-align: top">N/A</td>
    </tr>
    <tr>
        <td style="vertical-align: top">{{RDBMS-Name}}.indexCreateQuery</td>
        <td style="vertical-align: top; word-wrap: break-word">The template query for the <code>create index</code> operation in {{RDBMS-Name}}.</td>
        <td style="vertical-align: top"><b>H2</b>: CREATE INDEX {{TABLE_NAME}}_INDEX ON {{TABLE_NAME}} ({{INDEX_COLUMNS}})<br><b>MySQL</b>: CREATE INDEX {{TABLE_NAME}}_INDEX ON {{TABLE_NAME}} ({{INDEX_COLUMNS}})<br><b>Oracle</b>: CREATE INDEX {{TABLE_NAME}}_INDEX ON {{TABLE_NAME}} ({{INDEX_COLUMNS}})<br><b>Microsoft SQL Server</b>: CREATE INDEX {{TABLE_NAME}}_INDEX ON {{TABLE_NAME}} ({{INDEX_COLUMNS}}) {{TABLE_NAME}} ({{INDEX_COLUMNS}})<br><b>PostgreSQL</b>: CREATE INDEX {{TABLE_NAME}}_INDEX ON {{TABLE_NAME}} ({{INDEX_COLUMNS}})<br><b>DB2.*</b>: CREATE INDEX {{TABLE_NAME}}_INDEX ON {{TABLE_NAME}} ({{INDEX_COLUMNS}})</td>
        <td style="vertical-align: top">N/A</td>
    </tr>
    <tr>
        <td style="vertical-align: top">{{RDBMS-Name}}.recordInsertQuery</td>
        <td style="vertical-align: top; word-wrap: break-word">The template query for the <code>insert record</code> operation in {{RDBMS-Name}}.</td>
        <td style="vertical-align: top"><b>H2</b>: INSERT INTO {{TABLE_NAME}} VALUES ({{Q}})<br><b>MySQL</b>: INSERT INTO {{TABLE_NAME}} VALUES ({{Q}})<br><b>Oracle</b>: INSERT INTO {{TABLE_NAME}} VALUES ({{Q}})<br><b>Microsoft SQL Server</b>: INSERT INTO {{TABLE_NAME}} VALUES ({{Q}})<br><b>PostgreSQL</b>: INSERT INTO {{TABLE_NAME}} VALUES ({{Q}})<br><b>DB2.*</b>: INSERT INTO {{TABLE_NAME}} VALUES ({{Q}})</td>
        <td style="vertical-align: top">N/A</td>
    </tr>
    <tr>
        <td style="vertical-align: top">{{RDBMS-Name}}.recordUpdateQuery</td>
        <td style="vertical-align: top; word-wrap: break-word">The template query for the <code>update record</code> operation in {{RDBMS-Name}}.</td>
        <td style="vertical-align: top"><b>H2</b>: UPDATE {{TABLE_NAME}} SET {{COLUMNS_AND_VALUES}} {{CONDITION}}<br><b>MySQL</b>: UPDATE {{TABLE_NAME}} SET {{COLUMNS_AND_VALUES}} {{CONDITION}}<br><b>Oracle</b>: UPDATE {{TABLE_NAME}} SET {{COLUMNS_AND_VALUES}} {{CONDITION}}<br><b>Microsoft SQL Server</b>: UPDATE {{TABLE_NAME}} SET {{COLUMNS_AND_VALUES}} {{CONDITION}}<br><b>PostgreSQL</b>: UPDATE {{TABLE_NAME}} SET {{COLUMNS_AND_VALUES}} {{CONDITION}}<br><b>DB2.*</b>: UPDATE {{TABLE_NAME}} SET {{COLUMNS_AND_VALUES}} {{CONDITION}}</td>
        <td style="vertical-align: top">N/A</td>
    </tr>
    <tr>
        <td style="vertical-align: top">{{RDBMS-Name}}.recordSelectQuery</td>
        <td style="vertical-align: top; word-wrap: break-word">The template query for the <code>select record</code> operation in {{RDBMS-Name}}.</td>
        <td style="vertical-align: top"><b>H2</b>: SELECT * FROM {{TABLE_NAME}} {{CONDITION}}<br><b>MySQL</b>: SELECT * FROM {{TABLE_NAME}} {{CONDITION}}<br><b>Oracle</b>: SELECT * FROM {{TABLE_NAME}} {{CONDITION}}<br><b>Microsoft SQL Server</b>: SELECT * FROM {{TABLE_NAME}} {{CONDITION}}<br><b>PostgreSQL</b>: SELECT * FROM {{TABLE_NAME}} {{CONDITION}}<br><b>DB2.*</b>: SELECT * FROM {{TABLE_NAME}} {{CONDITION}}</td>
        <td style="vertical-align: top">N/A</td>
    </tr>
    <tr>
        <td style="vertical-align: top">{{RDBMS-Name}}.recordExistsQuery</td>
        <td style="vertical-align: top; word-wrap: break-word">The template query for the <code>check record existence</code> operation in {{RDBMS-Name}}.</td>
        <td style="vertical-align: top"><b>H2</b>: SELECT TOP 1 1 FROM {{TABLE_NAME}} {{CONDITION}}<br><b>MySQL</b>: SELECT 1 FROM {{TABLE_NAME}} {{CONDITION}}<br><b>Oracle</b>: SELECT COUNT(1) INTO existence FROM {{TABLE_NAME}} {{CONDITION}}<br><b>Microsoft SQL Server</b>: SELECT TOP 1 FROM {{TABLE_NAME}} {{CONDITION}}<br><b>PostgreSQL</b>: SELECT 1 FROM {{TABLE_NAME}} {{CONDITION}} LIMIT 1<br><b>DB2.*</b>: SELECT 1 FROM {{TABLE_NAME}} {{CONDITION}} FETCH FIRST 1 ROWS ONLY</td>
        <td style="vertical-align: top">N/A</td>
    </tr>
    <tr>
        <td style="vertical-align: top">{{RDBMS-Name}}.recordDeleteQuery</td>
        <td style="vertical-align: top; word-wrap: break-word">The query for the <code>delete record</code> operation in {{RDBMS-Name}}.</td>
        <td style="vertical-align: top"><b>H2</b>: DELETE FROM {{TABLE_NAME}} {{CONDITION}}<br><b>MySQL</b>: DELETE FROM {{TABLE_NAME}} {{CONDITION}}<br><b>Oracle</b>: DELETE FROM {{TABLE_NAME}} {{CONDITION}}<br><b>Microsoft SQL Server</b>: DELETE FROM {{TABLE_NAME}} {{CONDITION}}<br><b>PostgreSQL</b>: DELETE FROM {{TABLE_NAME}} {{CONDITION}}<br><b>DB2.*</b>: DELETE FROM {{TABLE_NAME}} {{CONDITION}}</td>
        <td style="vertical-align: top">N/A</td>
    </tr>
    <tr>
        <td style="vertical-align: top">{{RDBMS-Name}}.stringSize</td>
        <td style="vertical-align: top; word-wrap: break-word">This defines the length for the string fields in {{RDBMS-Name}}.</td>
        <td style="vertical-align: top"><b>H2</b>: 254<br><b>MySQL</b>: 254<br><b>Oracle</b>: 254<br><b>Microsoft SQL Server</b>: 254<br><b>PostgreSQL</b>: 254<br><b>DB2.*</b>: 254</td>
        <td style="vertical-align: top">N/A</td>
    </tr>
    <tr>
        <td style="vertical-align: top">{{RDBMS-Name}}.batchSize</td>
        <td style="vertical-align: top; word-wrap: break-word">This defines the batch size when operations are performed for batches of events.</td>
        <td style="vertical-align: top"><b>H2</b>: 1000<br><b>MySQL</b>: 1000<br><b>Oracle</b>: 1000<br><b>Microsoft SQL Server</b>: 1000<br><b>PostgreSQL</b>: 1000<br><b>DB2.*</b>: 1000</td>
        <td style="vertical-align: top">N/A</td>
    </tr>
    <tr>
        <td style="vertical-align: top">{{RDBMS-Name}}.batchEnable</td>
        <td style="vertical-align: top; word-wrap: break-word">This specifies whether <code>Update</code> and <code>Insert</code> operations can be performed for batches of events or not.</td>
        <td style="vertical-align: top"><b>H2</b>: true<br><b>MySQL</b>: true<br><b>Oracle</b>: true<br><b>Microsoft SQL Server</b>: true<br><b>PostgreSQL</b>: true<br><b>DB2.*</b>: true</td>
        <td style="vertical-align: top">N/A</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
@Store(type="rdbms", jdbc.url="jdbc:mysql://localhost:3306/das", username="root", password="root" , jdbc.driver.name="org.h2.Driver",field.length="symbol:100")
@PrimaryKey("symbol")@Index("volume")define table StockTable (symbol string, price float, volume long);
```
<p style="word-wrap: break-word">The above example creates an event table named <code>StockTable</code> on the DB if it does not already exist (with 3 attributes named <code>symbol</code>, <code>price</code>, and <code>volume</code> of the types types <code>string</code>, <code>float</code> and <code>long</code> respectively). The connection is made as specified by the parameters configured for the '@Store' annotation. The <code>symbol</code> attribute is considered a unique field, and a DB index is created for it.</p>

